<?php

namespace Drupal\feeds_xsltparser\Feeds\Parser\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\feeds\Plugin\Type\ExternalPluginFormBase;
use Drupal\feeds_xsltparser\XSLTPipeRepository;

/**
 * The configuration form for the XSLT pipeline parser.
 */
class XsltPipelineParserForm extends ExternalPluginFormBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['xslt_dir'] = [
      '#type' => 'textfield',
      '#title' => $this->t('XSLT Pipelines Path'),
      '#description' => $this->t('A path on the server to the directory where your XSLT pipelines are stored'),
      '#size' => 60,
      '#default_value' => $this->plugin->getConfiguration('xslt_dir'),
    ];

    $xslt_dir = $this->configGetXsltDirectory();
    if ($xslt_dir === FALSE) {
      $count = 0;
    }
    else {
      $repository = new XSLTPipeRepository($xslt_dir);
      $config = $repository->config();
      $fieldmap = $repository->extractFieldmap($config);
      $fieldLabels = array_keys($fieldmap['mapper']['items']);
      $count = count($fieldLabels);
    }

    if ($count == 0) {
      $form['xslt_help'] = [
        '#type' => 'markup',
        '#markup' => $this->t('No pipelines found in XSLT pipeline path.') . '<br/>' .
         $this->t('In order to create pipelines upload your XSLT files into the specified directory. If you place folders containing XSLT files under the XSLT pipeline path, the stylesheets will be executed in alphabetic order, each operating on the result of its predecessor.') . '<br/>' .
         $this->t('Each pipeline is exposed as a source field in the mapping section'),
      ];
    }
    else {
      $items = $fieldmap['mapper']['items'];
      $pipes = [];
      foreach ($fieldLabels as $label) {
        $pipes[$label] = $this->describeFieldExtractor($items[$label], $xslt_dir);
      }

      $form['xslt_help'] = [
        '#theme' => 'feeds_xsltparser_pipelist',
        '#pipes' => $pipes,
      ];
    }

    return $form;
  }

  /**
   * Returns a string describing the given field extractor fragemnt from a
   * pipeline config. Typically you use this to generate help text. See
   * FeedsXSLTPipelineParser:configForm for an example of how this function
   * is called.
   */
  private function describeFieldExtractor($item, string $base_dir) {
    $reloffset = mb_strlen($base_dir) + 1;

    $output = $this->t('Unknown extractor type');
    if ($item['type'] == 'XSLTMap' && $item['mapper']['type'] == 'XSLTRepeat') {
      $extractors = substr_replace(
        $item['mapper']['pipe']['stylesheets'],
        '',
        0,
        $reloffset
      );
      $counters = substr_replace(
        $item['mapper']['counter']['stylesheets'],
        '',
        0,
        $reloffset
      );
      $output = $this->t('Extract multiple field values by applying @extracttext %extractors repeatedly. The XSLT parameter %paramname is set to the current iteration (0 in the first pass, 1 in the second, etc.). The number of repetitions is determined using @countertext %counters, which must return an integer value.', [
        '@extracttext' => $this->formatPlural(count($extractors), 'stylesheet', 'stylesheet chain comprised of the extractors'),
        '%paramname' => $item['paramKey'],
        '%extractors' => implode(', ', $extractors),
        '@countertext' => $this->formatPlural(count($counters), 'stylesheet', 'stylesheet chain comprised of the counters'),
        '%counters' => implode(', ', $counters),
      ]);

      if (isset($item['preproc'])) {
        $preprocs = substr_replace($item['preproc']['stylesheets'], '', 0, $reloffset);
        $output .= ' ';
        $output .= $this->t('Input gets preprocessed using @filetext %files, counting and value extraction is done on the result obtained form the preprocessor.', [
          '@filetext' => $this->formatPlural(count($preprocs), 'stylesheet', 'stylesheet chain comprised of the files'),
          '%files' => implode(', ', $preprocs),
        ]);
      }
    }
    elseif ($item['type'] == 'XSLTChain') {
      $extractors = substr_replace($item['stylesheets'], '', 0, $reloffset);
      $output = $this->t('Extract one field value using @filetext %files.', [
        '@filetext' => $this->formatPlural(count($extractors), 'stylesheet', 'stylesheet chain comprised of the files'),
        '%files' => implode(', ', $extractors),
      ]);
    }

    return $output;
  }

  private function configGetXsltDirectory() {
    $xslt_dir = $this->plugin->getConfiguration('xslt_dir');
    if ($xslt_dir[0] != '/') {
      $xslt_dir = \Drupal::root() . '/' . $xslt_dir;
    }

    return is_dir($xslt_dir) ? $xslt_dir : FALSE;
  }

}
