<?php

namespace Drupal\Tests\feeds_xsltparser\Unit;

use Drupal\feeds_xsltparser\LibXMLException;
use Drupal\feeds_xsltparser\XSLTChain;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\feeds_xsltparser\XSLTChain
 * @group feeds_xsltparser
 */
class XSLTChainTestCase extends UnitTestCase {
  /**
   * @var \XSLTProcessor
   */
  private $goodtotextproc;

  /**
   * @var \XSLTProcessor
   */
  private $goodtoxmlproc;

  /**
   * @var \XSLTProcessor
   */
  private $goodparamproc;

  /**
   * @var \XSLTProcessor
   */
  private $badproc;

  /**
   * @var string
   */
  private $goodmarkup;

  /**
   * @var string
   */
  private $badmarkup;

  /**
   * @var bool
   */
  private $xsl_available;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->xsl_available = extension_loaded('xsl');

    if (!$this->xsl_available) {
      trigger_error('XSL Extension is not available on this system. Skipping this test.', E_USER_NOTICE);
      return;
    }

    $goodxsltoxml =
      '<?xml version="1.0" encoding="UTF-8"?>' .
      '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">' .
      '<xsl:output method="xml" encoding="UTF-8"/>' .
      '<xsl:template match="/">' .
      '  <alsogood><xsl:value-of select="."/></alsogood>' .
      '</xsl:template>' .
      '</xsl:stylesheet>';
    $this->goodtoxmlproc = new \XSLTProcessor();
    $doc = new \DOMDocument();
    $doc->loadXML($goodxsltoxml);
    $this->goodtoxmlproc->importStylesheet($doc);

    $goodxsltotext =
      '<?xml version="1.0" encoding="UTF-8"?>' .
      '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">' .
      '<xsl:output method="text" encoding="UTF-8"/>' .
      '<xsl:template match="/">' .
      '<xsl:value-of select="."/>' .
      '</xsl:template>' .
      '</xsl:stylesheet>';
    $this->goodtotextproc = new \XSLTProcessor();
    $doc = new \DOMDocument();
    $doc->loadXML($goodxsltotext);
    $this->goodtotextproc->importStylesheet($doc);

    $paramxslt =
      '<?xml version="1.0" encoding="UTF-8"?>' .
      '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">' .
      '<xsl:output method="text" encoding="UTF-8"/>' .
      '<xsl:param name="testparam"/>' .
      '<xsl:template match="/">' .
      '<xsl:value-of select="$testparam"/>' .
      '</xsl:template>' .
      '</xsl:stylesheet>';
    $this->goodparamproc = new \XSLTProcessor();
    $doc = new \DOMDocument();
    $doc->loadXML($paramxslt);
    $this->goodparamproc->importStylesheet($doc);

    $badxslt =
      '<?xml version="1.0" encoding="UTF-8"?>' .
      '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">' .
      '<xsl:output method="text" encoding="UTF-8"/>' .
      '<xsl:template match="/">' .
      '<xsl:message terminate="yes">Simulate runtime error</xsl:message>' .
      '</xsl:template>' .
      '</xsl:stylesheet>';
    $this->badproc = new \XSLTProcessor();
    $doc = new \DOMDocument();
    $doc->loadXML($badxslt);
    $this->badproc->importStylesheet($doc);

    $this->goodmarkup =
      '<?xml version="1.0" encoding="UTF-8"?>' .
      '<good>Heck, it works!</good>';

    $this->badmarkup =
      '<?xml version="1.0" encoding="UTF-8"?>' .
      '<really>not parsable</bad>';
  }

  /**
   * The original markup should be returned by the XSLTChain if no processor
   * was added to the chain.
   */
  public function testChainWithoutAnyProcessorShouldReturnOriginalMarkup() {
    if (!$this->xsl_available) {
      return;
    }

    $chain = new XSLTChain();
    $result = $chain->run($this->goodmarkup);

    $this->assertEquals($this->goodmarkup, $result);
  }

  /**
   * Test wether the result of the XSLT transformation with one single
   * stylesheet is correct.
   */
  public function testChanWithOneStylesheetShouldReturnCorrectResult() {
    if (!$this->xsl_available) {
      return;
    }

    $chain = new XSLTChain();
    $chain->addProcessor($this->goodtotextproc);
    $result = $chain->run($this->goodmarkup);

    $this->assertEquals('Heck, it works!', $result);
  }

  /**
   * Test wether the result of the XSLT transformation with two stylesheet
   * applied one on top of the results of the other is correct.
   */
  public function testChanWithMultipleStylesheetsShouldReturnCorrectResult() {
    if (!$this->xsl_available) {
      return;
    }

    $chain = new XSLTChain();
    $chain->addProcessor($this->goodtoxmlproc);
    $chain->addProcessor($this->goodtotextproc);
    $result = $chain->run($this->goodmarkup);

    $this->assertEquals('Heck, it works!', $result);
  }

  /**
   * Test wether the result of the XSLT transformation with one single
   * stylesheet is correct.
   */
  public function testChanWithParam() {
    if (!$this->xsl_available) {
      return;
    }

    $chain = new XSLTChain();
    $chain->addProcessor($this->goodparamproc);

    $result = $chain->run($this->goodmarkup, ['testparam' => 'first']);
    $this->assertEquals('first', $result);

    $result = $chain->run($this->goodmarkup, ['testparam' => 'second']);
    $this->assertEquals('second', $result);
  }

  /**
   * Ensure that an exception is thrown when the markup is incorrect.
   */
  public function testChainExceptionOnInvalidMarkup() {
    if (!$this->xsl_available) {
      return;
    }

    $chain = new XSLTChain();
    $chain->addProcessor($this->goodtotextproc);

    $this->expectException(LibXMLException::class);
    $chain->run($this->badmarkup);
  }

  /**
   * Ensure that an exception is thrown when a runtime error occures in the
   * XSLT transformation.
   */
  public function testChainExceptionOnXSLTRuntimeError() {
    if (!$this->xsl_available) {
      return;
    }

    $chain = new XSLTChain();
    $chain->addProcessor($this->badproc);

    $this->expectException(LibXMLException::class);
    $chain->run($this->goodmarkup);
  }

  /**
   * Ennsure that an XSLTChain with a stylesheet loaded from a given path
   * produces the expected results.
   */
  public function testAddProcessorFromPath() {
    if (!$this->xsl_available) {
      return;
    }

    $path = __DIR__ . '/../../fixture/goodtotext.xslt';
    $chain = new XSLTChain();
    $chain->addProcessorFromPath($path);

    $result = $chain->run($this->goodmarkup);

    $this->assertEquals('Heck, it works!', $result);
  }

  /**
   * When attempting to load an XML document which does not contain an XSLT
   * stylesheet, an exception should be thrown.
   */
  public function testAddProcessorFromPathWithInvalidStylesheet() {
    if (!$this->xsl_available) {
      return;
    }

    $path = __DIR__ . '/../../fixture/noxsl.xslt';
    $chain = new XSLTChain();

    $this->expectException(LibXMLException::class);
    $chain->addProcessorFromPath($path);
  }

  /**
   * When attempting to load a stylesheet from a path which does not exist.
   */
  public function testAddProcessorFromPathFromNonExistingPath() {
    if (!$this->xsl_available) {
      return;
    }

    $path = __DIR__ . '/../../fixture/path-does-not-exist';
    $chain = new XSLTChain();

    $this->expectException(LibXMLException::class);
    $chain->addProcessorFromPath($path);
  }

  /**
   * When attempting to load an invalid XML document as an XSLT stylesheet, an
   * exception should be thrown.
   */
  public function testAddProcessorFromPathWithInvalidXML() {
    if (!$this->xsl_available) {
      return;
    }

    $path = __DIR__ . '/../../fixture/invalidxml.xslt';
    $chain = new XSLTChain();

    $this->expectException(LibXMLException::class);
    $chain->addProcessorFromPath($path);
  }

}
